;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************
; Functions for GNUnilink - these have to be placed at start of code as the 
; PIC12 series can only 'Call' to lower addresses

;******************************************************************************
; Lookup table for serial output - defines the output strings.
; Must not cross a page boundary!

	IFDEF SERIAL_HARD	
        IFDEF SERIAL_OUTPUT
 
serial_command_table:
	movf	command_offset,0		; Move command offset to W
	addwf	PCL,F				; Add offset to program counter

command_start:
	retlw	'P'
	retlw	'L'
	retlw	'A'
	retlw	'Y'
	retlw	'\n'
	retlw	0

command_pause:
	retlw	'P'
	retlw	'A'
	retlw	'U'
	retlw	'S'
	retlw	'E'
	retlw	'\n'
	retlw	0

command_stop:
	retlw	'S'
	retlw	'T'
	retlw	'O'
	retlw	'P'
	retlw	'\n'
	retlw	0

command_forward:
	retlw	'F'
	retlw	'F'
	retlw	'\n'
	retlw	0

command_rewind:
	retlw	'R'
	retlw	'W'
	retlw	'\n'
	retlw	0

command_next_track:
	retlw	'N'
	retlw	'E'
	retlw	'X'
	retlw	'T'
	retlw	'\n'
	retlw	0

command_previous_track:
	retlw	'P'
	retlw	'R'
	retlw	'E'
	retlw	'V'
	retlw	'I'
	retlw	'O'
	retlw	'U'
	retlw	'S'
	retlw	'\n'
	retlw	0

	IFNDEF	DISKDIRECT
command_next_disk:
	retlw	'N'
	retlw	'X'
	retlw	'T'
	retlw	'_'
	retlw	'L'
	retlw	'I'
	retlw	'S'
	retlw	'T'
	retlw	'\n'
	retlw	0

command_previous_disk:
	retlw	'P'
	retlw	'R'
	retlw	'V'
	retlw	'_'
	retlw	'L'
	retlw	'I'
	retlw	'S'
	retlw	'T'
	retlw	'\n'
	retlw	0
	ELSE
command_direct_disk:
	retlw	'L'
	retlw	'I'
	retlw	'S'
	retlw	'T'
	retlw	'_'
	retlw	255			; replace this character
	retlw	'\n'
	retlw	0
	ENDIF

	IFDEF	TRACKMODES
command_intro:
	retlw	'I'
	retlw	'N'
	retlw	'T'
	retlw	'R'
	retlw	'O'
	retlw	'\n'
	retlw	0

command_shuffle:
	retlw	'S'
	retlw	'H'
	retlw	'U'
	retlw	'F'
	retlw	'F'
	retlw	'\n'
	retlw	0

command_repeat:
	retlw	'R'
	retlw	'E'
	retlw	'P'
	retlw	'E'
	retlw	'A'
	retlw	'T'
	retlw	'\n'
	retlw	0
	ENDIF

command_release:
	retlw	'R'
	retlw	'E'
	retlw	'L'
	retlw	'\n'
	retlw	0

	ENDIF ;SERIAL_OUTPUT
	ELSE  ;SERIAL_HARD
	;; serial soft commands are 1 char long
        IFDEF SERIAL_OUTPUT
 
serial_command_table:
	movf	command_offset,0		; Move command offset to W
	addwf	PCL,F				; Add offset to program counter

command_start:
	retlw	0x41    ; start
command_pause:
	retlw	0x42     ; pause (tuner)
command_stop:
	retlw	0x43     ; stop
command_next_track:
	retlw	0x44
command_previous_track:
	retlw	0x45
	IFNDEF	DISKDIRECT
command_next_disk:
	retlw	0x46
command_previous_disk:
	retlw	0x47
	ELSE
command_direct_disk:
	retlw	'L'
	ENDIF

	IFDEF	TRACKMODES
command_intro:
	retlw	0x48
command_shuffle:
	retlw	0x49
command_repeat:
	retlw	0x4a
	ENDIF
command_release:
	retlw	0x4b
command_forward:
	retlw	0x4c

command_rewind:
	retlw	0x4d

	ENDIF


	ENDIF
;******************************************************************************
; Send a Slave Break - tells the Master that you have something to tell them...

; In this function GNUnilink looks for a period when the data bus is low for 
; more than 6ms and then waits for it to be high for 2ms. Once this is seen it 
; knows were in the middle of an unused 'slave phase' and it then forces the 
; bus low for 4ms.

; NOTE - whilst GNUnilink is in this state it is unable to receive other 
; commands from the master
; sophana:this is the problem why I removed that function.

IssueSlaveBreak:
        bsf     SophFlag , DoBreak
        return

;	movlw	60				; 6ms (60 * 100us)
;	movwf	Count
;
;BreakLoop1:
;	movlw	delay100us			; 100us 
;	movwf	Count2
;
;BreakLoop2:
;	btfsc	OUT_PORT, DataBit		; Data low ('Master phase')
;	goto	IssueSlaveBreak
;
;	goto	BreakLoop3			; NOP
;BreakLoop3:
;
;	decfsz	Count2, 1			; Reduce Count2
;	goto	BreakLoop2
;
;	decfsz	Count, 1			; Reduce count 
;	goto	BreakLoop1
;
;	IFDEF	SERIAL_HARD
;	bcf	INTCON, GIE
;	ENDIF
;
;BreakLoop4:
;	IFDEF	NASTY_HACK
;	movlw	40				; rely on timing. 
;	ELSE
;	movlw	20				; 2ms (20 * 100us)
;	ENDIF
;	movwf	Count
;
;BreakLoop5:
;	movlw	delay100us
;	movwf	Count2
;
;BreakLoop6:
;	IFNDEF	NASTY_HACK
;	btfss	OUT_PORT, DataBit		; Data high for 2ms 
;	goto	BreakLoop4			; (moving into 'Slave Phase')
;	ENDIF
;
;	goto	BreakLoop7			; NOP
;BreakLoop7:
;
;	decfsz	Count2, 1			; Reduce Count2
;	goto	BreakLoop6
;
;	decfsz	Count, 1			; Reduce count 
;	goto	BreakLoop5
;
;	IFDEF PIC_12
;	movlw	0xfe				; Enable data pin
;	tris	0
;	ENDIF
;
;	IFDEF PIC_16
;	bsf	STATUS, RP0			; Enable data pin as output
;	bcf	OUT_ENA, DataBit
;	bcf	STATUS, RP0
;	ENDIF
;
;	bcf	OUT_PORT, DataBit		; Force Data low.... 4ms
;
;	movlw	40
;	movwf	Count
;
;BreakLoop9:
;	movlw	delay100us
;	movwf	Count2
;
;BreakLoop10:
;	goto	BreakLoop11			; 2 * NOP
;BreakLoop11:
;	goto	BreakLoop12			; 2 * NOP
;BreakLoop12:
;
;	decfsz	Count2, 1
;	goto	BreakLoop10
;
;	decfsz	Count, 1
;	goto	BreakLoop9
;
;	IFDEF PIC_12
;	movlw	0xff				; Tristate data pin
;	tris	0
;	ENDIF
;
;	IFDEF PIC_16
;	bsf	STATUS, RP0			; Tristate data pin
;	bsf	OUT_ENA, DataBit
;	bcf	STATUS, RP0
;	ENDIF
;
;	IFDEF	SERIAL_HARD
;	bsf	INTCON, GIE
;	ENDIF
;
;	return

;*******************************************************
; sophana: wait for w*100us
DoWait:
	movwf	Count

WaitLoop9:
	movlw	delay100us
	movwf	Count2

WaitLoop10:
	goto	WaitLoop11			; 2 * NOP
WaitLoop11:
	goto	WaitLoop12			; 2 * NOP
WaitLoop12:

	decfsz	Count2, 1
	goto	WaitLoop10

	decfsz	Count, 1
	goto	WaitLoop9
    return

;******************************************************************************
; Receive a byte - received byte is placed in W register.
; Input is sampled between 1 cycle before and 3 cycles after Clock rising edge,
; depending on the asynchronous alignment with the Master

ReadByte:
	clrf	DataVal

	movlw	0x08
	movwf	Count

	bcf	STATUS, C			; Ensure carry bit is 0

ReadLoop1:
	rlf	DataVal, 1
	
ReadLoop2:
	btfss	OUT_PORT, ClockBit		; Wait for clock going high
	goto	ReadLoop2

ReadLoop3:
	IFDEF	SERIAL_HARD
	bcf	INTCON, GIE
	ENDIF

	movf	OUT_PORT, 0			; Sample input

	btfsc	OUT_PORT, ClockBit		; Wait for clock going low
	goto	ReadLoop3

	movwf	Count2				; (inverted) input
	btfss	Count2, DataBit
	bsf	DataVal, 0
	
	decfsz	Count, 1			; Do next bit, if any left...
	goto	ReadLoop1

	IFDEF	SERIAL_HARD
	bsf	INTCON, GIE
	ENDIF

	IFDEF   SERIAL_LOG
	IFDEF	BUS_ON_LOG
	movf	SophFlag,w
	xorwf	PORTA,w
	andlw	1<<BusOnBit
	btfsc	STATUS,Z
	goto	ReadByteSend
	movf	SophFlag,w
	andlw	1<<BusOnBit
	addlw	0x60
	call	Send
	movlw	1<<BusOnBit
	xorwf	SophFlag,f	
ReadByteSend:
	ENDIF	; BUS_ON_LOG
	movf	DataVal, 0
	call    Send
	ENDIF
	movf	DataVal, 0
	return

;******************************************************************************
; Send a byte - byte to ouput is in W register.

; This is the piece of code which is the most time critcal.
; Worst case the outpit bit is set a maximum of 7 cycles after Clock goes High
; and held until the Clock goes high again

; this hopefully gives enough time for the master to register it.... a LOT 
; dodgy on slow systems.

; We may have to reverse the logic on this so that the output is set before the 
; Clock goes high and changed after the Clock goes low. This will improve the i
; set up, but reduce the hold time.

SendByte:
	movwf	DataVal				; Store byte to output
	IFDEF   SERIAL_LOG
	call    Send
	movf	DataVal, 0
	ENDIF

	addwf	Checksum, 1			; Compute Checksum

	IFDEF PIC_12
	movlw	0xfe				; Enable data pin
	tris	0
	ENDIF

	IFDEF PIC_16
	bsf	STATUS, RP0			; Enable data pin
	bcf	OUT_ENA, DataBit
	bcf	STATUS, RP0
	ENDIF

	movlw	0x08
	movwf	Count

SendLoop1:
	bcf	OUT_PORT, DataBit		; remember it's INVERTED

SendLoop2:
	btfss	OUT_PORT, ClockBit		; Wait for clock going high
	goto	SendLoop2

	IFDEF	SERIAL
	bcf	INTCON, GIE
	ENDIF
	
	btfss	DataVal, 7
	bsf	OUT_PORT, DataBit

	rlf	DataVal, 1

SendLoop3:
	btfsc	OUT_PORT, ClockBit		; Wait for clock going low
	goto	SendLoop3

	decfsz	Count, 1			; Do next bit, if any left...
	goto	SendLoop1

	goto	SendLoop4			; just a little data hold
SendLoop4:

	IFDEF PIC_12
	movlw	0xff				; Tristate data pin
	tris	0
	ENDIF

	IFDEF PIC_16
	bsf	STATUS, RP0			; Trisate data pin
	bsf	OUT_ENA, DataBit
	bcf	STATUS, RP0
	ENDIF
	
	IFDEF	SERIAL
	bsf	INTCON, GIE
	ENDIF

	return

;******************************************************************************
; Bit frig - works out which bit to set in the response to Master Poll

; W register is input of which stage you are on (0x00, 0x20, 0x30, 0x40 etc)
; and is returned with the byte to write (0x00 if wrong stage).

Bit_Frig:
	xorwf	MyBit, 0
	andlw	0xe0				; Strip off low bits

	btfsc	STATUS, Z			; Do we have a hit?
	goto	Bit_Frig_Hit

	movlw	0x00
	return

Bit_Frig_Hit:
	btfss	MyBit, 4			; Do we need to swap nybbles?
	goto	Bit_Frig_Swap

	movf	MyBit, 0
	andlw	0x0F
	return

Bit_Frig_Swap:
	swapf	MyBit, 0
	andlw	0xF0
	return

	IFDEF SERIAL_SOFT
    ;; **************************************************************
	;; code for asynchronous serial communication for 16f84 (no uart)
	;; taken from a french pic course from bigonoff


BANK0	macro
		bcf	STATUS,RP0			; passer banque0
		endm

BANK1	macro
		bsf	STATUS,RP0			; passer banque1
		endm

	
;*********************************************************************
;                      TEMPORISATION                                 *
;*********************************************************************
;---------------------------------------------------------------------
; temp_1bd: initialise tmr0 pour que la temporisation soit gale 
;           l'quivalent d'un bit et demi, soit 46+93 incrmentations
;           de tmr0 - le temps ncessaire pour arriver dans la routine
; temp_1b : Attend que l'cart entre la prcdente tempo et la tempo
;           actuelle soit de 1 bit, soit 93 instructions
;---------------------------------------------------------------------

temp_1bd
	movlw	(SERIAL_BIT_TIME/2)-6		; prparer valeur tmr0
	movwf	TMR0				; initialiser tmr0
	call	temp_suite			; et attendre 1/2 bit

temp_1b
	movlw	SERIAL_BIT_TIME			; cart entre 2 bits
	addwf	TMR0,f				; ajouter  la valeur actuelle

temp_suite
	bcf	INTCON,T0IF			; effacer flag	
temp_wait
	btfss	INTCON,T0IF			; attendre dbordement timer
	goto	temp_wait			; pas fini, attendre
	return						; et sortir
;*********************************************************************
;                 Envoi d'un octet vers le lecteur de carte          *
;*********************************************************************
;---------------------------------------------------------------------
; envoie l'octet contenu dans le registre w vers le lecteur de carte
;---------------------------------------------------------------------
Send
	btfsc	SophFlag , DoPulse		; the archos does not support chars
	return					; while being powered on.
	movwf	caract				; Sauver caractre  envoyer
	IFDEF	SERIAL_OPEN_COL
	bcf     SERIAL_PORT	       		; port srie a 0
	ELSE
	BANK1
	bcf	SERIAL_PORT			;  drive serial port pin
	BANK0
	ENDIF
	SERIAL_0				; envoyer 0 : start-bit

	; send start-bit
	; -----------------
	movlw	SERIAL_BIT_TIME			; prparer valeur tmr0
	movwf	TMR0				; initialiser tmr0

	movlw	8				; pour 8=(9-1) bits  envoyer
	movwf	cmptbts				; dans compteur de bits
	call	temp_suite			; attendre 1 bit


	; envoyer 8 bits de data
	; ----------------------
Send_loop
	rrf	caract , f			; dcaler caractre, b0 dans carry
	btfss   STATUS , C
	goto    Send_0
	SERIAL_1				; envoyer 1 
	call	temp_1b				; attente entre 1 bits
	decfsz	cmptbts , f			; dcrmenter compteur de bits
	goto	Send_loop			; pas dernier, suivant
	goto    Send_End
Send_0
	SERIAL_0				; envoyer 0
	call	temp_1b				; attente entre 1 bits
	decfsz	cmptbts , f			; dcrmenter compteur de bits
	goto	Send_loop			; pas dernier, suivant

Send_End
						; envoyer 1 stop-bits
						; -------------------
	SERIAL_1				; repasser en entre (et niveau haut)	
	call	temp_1b				; attendre temps entre 2 bits
	return					; et retour

;*********************************************************************
;              Rception d'un octet provenant du maitre              *
;*********************************************************************
;---------------------------------------------------------------------
; Caractre lu dans W. La parit pas n'est pas vrifie
;---------------------------------------------------------------------
Receive
				; attendre dbut start-bit
				; ------------------------

	BANK0					; passer banque0
	btfsc	SERIAL_PORT			; Tester si start bit arriv
	goto   Receive				; non, attendre

		; se positionner sur le milieu du 1er bit utile
		; ---------------------------------------------

	call	temp_1bd			; attendre 1bit et demi

				; rception du caractre
				; ----------------------

	movlw	0x8				; pour 8 bits
	movwf	cmptbts				; dans compteur de bits

Recloop
	bcf	STATUS,C			; Carry = 0
	btfsc	SERIAL_PORT			; tester si bit = 0
	bsf	STATUS,C			; Carry = bit reu
	rrf	caract,f			; faire entrer le bit par la gauche
	call	temp_1b				; attendre milieu caractre suivant
	decfsz	cmptbts,f			; dcrmenter compteur de bits
	goto	Recloop				; pas dernier, suivant


	; on pointe maintenant sur moiti du bit de parit
	; reste donc  attendre au moins 1.5 bits
	; ------------------------------------------------

	;call	temp_1bd			; Attendre 1,5 bit
	movf	caract,w			; charger caractre lu
	return					; et retour

	ENDIF
	
